/*
 * IIIF Tsukushi Viewer - Firebase authentication registration plugin
 * http://codh.rois.ac.jp/software/iiif-tsukushi-viewer/
 *
 * Copyright 2024 Center for Open Data in the Humanities, Research Organization of Information and Systems
 * Released under the MIT license
 *
 * Core contributor: Jun HOMMA (@2SC1815J)
 */
(function() {
    'use strict';
    var pluginHost = typeof iiifViewer !== 'undefined' ? iiifViewer : void 0;
    var err;

    //依存関係の確認
    if (!pluginHost) {
        err = new Error(); logError('Plugin host not found.', err.lineNumber);
        return;
    }
    if (!$.isFunction(pluginHost.updateFirebaseIdToken)) {
        err = new Error(); logError('Plugin host not compatible.', err.lineNumber);
        return;
    }

    //----------------------------------------------------------------------
    //Firebase認証
    var idToken = null;
    var idTokenResult = null;
    if (typeof firebase !== 'undefined' && firebase) {
        updateIdTokenCurrentUser();
        firebase.auth().onAuthStateChanged(function(user) {
            updateIdToken(user);
        });
    }
    function updateIdToken(user) {
        idToken = null;
        idTokenResult = null;
        if (user) {
            var currentUser = firebase.auth().currentUser;
            if (currentUser) {
                currentUser.getIdTokenResult(true).then(function(idTokenResult_) {
                    idToken = idTokenResult_.token;
                    idTokenResult = idTokenResult_;
                    pluginHost.updateFirebaseIdToken(idToken);
                }).catch(function(error) {
                    err = new Error(); logError(error.message, err.lineNumber);
                });
            }
        } else {
            //ログアウトしたとき
            pluginHost.updateFirebaseIdToken(idToken);
        }
    }
    function updateIdTokenCurrentUser() {
        updateIdToken(firebase.auth().currentUser);
    }
    // checkIntervalMinutes は minutesBeforeExpirationToRefresh より小さい値に設定すること
    const checkIntervalMinutes = 10;
    const minutesBeforeExpirationToRefresh = 15;
    setInterval(function() {
        if (idTokenResult) {
            const expirationTime = dayjs(idTokenResult.expirationTime);
            const expiresIn = expirationTime.diff(dayjs(new Date()), 'second');
            if (expiresIn <= minutesBeforeExpirationToRefresh * 60) {
                updateIdTokenCurrentUser();
            }
        }
    }, checkIntervalMinutes * 60 * 1000);

    function logError(message, lineNumber) {
        if (window.console) {
            var pluginName = 'Firebase authentication registration plugin';
            var msg = (pluginHost ? pluginHost.getName() : 'IIIF Curation Platform') + ' (' + pluginName + '): ';
            var details = [];
            if (message) {
                details.push(message);
            }
            if (lineNumber) {
                details.push('line: ' + lineNumber);
            }
            if (details.length > 0) {
                msg += details.join(', ');
            }
            console.log(msg); // eslint-disable-line no-console
        }
    }
})();